/* autogenerated, do not edit */
#include <X11/Intrinsic.h>
String fallback_resources[] = {
"Xforge*fontList:					"
"	-*-helvetica-bold-r-normal--*-120-*-*-*-*-*",
"Xforge*foreground:					Black",
"Xforge*background:					gray",
"Xforge*canvas.background:				LightGray",
"Xforge*canvas.height:					500",
"Xforge*canvas.width:					700",
"Xforge*quitDialog.dialogTitle:				Xforge: Quit",
"Xforge*quitDialog.messageString:			Really quit?",
"Xforge*clearDialog.dialogTitle:				Xforge: Clear",
"Xforge*clearDialog.messageString:			Really clear?",
"Xforge*addwsDialog.dialogTitle:				Xforge: Add workspace",
"Xforge*addwsDialog.selectionLabelString:		"
"	Add workspace samples...",
"Xforge*closeDialog.messageString:			Really close?",
"Xforge*paramDialog.verticalSpacing:			10",
"Xforge*paramDialog.horizontalSpacing:			10",
"Xforge*paramDialog.autoUnmanage:			False",
"Xforge*paramDialog.formRate.rateLabel.labelString:	Sampling rate:",
"Xforge*paramDialog.formChan.chanLabel.labelString:	Channels:",
"Xforge*paramDialog.formChan.channels.ch1.labelString:	1",
"Xforge*paramDialog.formChan.channels.ch2.labelString:	2",
"Xforge*paramDialog.formBits.bitsLabel.labelString:	Sample bits:",
"Xforge*paramDialog.formBits.bits.bits8.labelString:	8",
"Xforge*paramDialog.formBits.bits.bits16.labelString:	16",
"Xforge*paramDialog.formBits.bits.bits24.labelString:	24",
"Xforge*paramDialog.formBits.bits.bits32.labelString:	32",
"Xforge*paramDialog.formFmt.fmtLabel.labelString:	File format:",
"Xforge*paramDialog.formFmt.fmt.spacing:			0",
"Xforge*paramDialog.formFmt.fmt.fmtRaw.labelString:	Raw",
"Xforge*paramDialog.formFmt.fmt.fmtSun.labelString:	Sun",
"Xforge*paramDialog.formFmt.fmt.fmtRIFF.labelString:	RIFF",
"Xforge*paramDialog.formFmt.fmt.fmtRIFX.labelString:	RIFX",
"Xforge*paramDialog.formFmt.fmt.fmtAIFF.labelString:	AIFF",
"Xforge*paramDialog.formFmt.fmt.fmtAIFC.labelString:	AIFF-C",
"Xforge*paramDialog.apply.labelString:			Apply",
"Xforge*paramDialog.apply.marginHeight:			4",
"Xforge*paramDialog.apply.marginWidth:			4",
"Xforge*paramDialog.dismiss.labelString:			Dismiss",
"Xforge*paramDialog.dismiss.marginHeight:		4",
"Xforge*paramDialog.dismiss.marginWidth:			4",
"Xforge*paramDialog.help.labelString:			Help",
"Xforge*paramDialog.help.marginHeight:			4",
"Xforge*paramDialog.help.marginWidth:			4",
"Xforge*resampleDialog.verticalSpacing:			10",
"Xforge*resampleDialog.horizontalSpacing:		10",
"Xforge*resampleDialog.autoUnmanage:			False",
"Xforge*resampleDialog.rowCol.rateLabel.labelString:	Sampling rate:",
"Xforge*resampleDialog.rowCol.wlenLabel.labelString:	Window length:",
"Xforge*resampleDialog.rowCol.wfuncLabel.labelString:	Window function:",
"Xforge*resampleDialog.rowCol.wfunc.rect.labelString:	Rectangular",
"Xforge*resampleDialog.rowCol.wfunc.bart.labelString:	Bartlett",
"Xforge*resampleDialog.rowCol.wfunc.black.labelString:	Blackman",
"Xforge*resampleDialog.rowCol.wfunc.hamming.labelString:	Hamming",
"Xforge*resampleDialog.rowCol.wfunc.hanning.labelString:	Hanning",
"Xforge*resampleDialog.apply.labelString:		Apply",
"Xforge*resampleDialog.apply.marginHeight:		4",
"Xforge*resampleDialog.apply.marginWidth:		4",
"Xforge*resampleDialog.dismiss.labelString:		Dismiss",
"Xforge*resampleDialog.dismiss.marginHeight:		4",
"Xforge*resampleDialog.dismiss.marginWidth:		4",
"Xforge*resampleDialog.help.labelString:			Help",
"Xforge*resampleDialog.help.marginHeight:		4",
"Xforge*resampleDialog.help.marginWidth:			4",
"Xforge*monoDialog.verticalSpacing:			10",
"Xforge*monoDialog.horizontalSpacing:			10",
"Xforge*monoDialog.autoUnmanage:				False",
"Xforge*monoDialog.left.labelString:			Left",
"Xforge*monoDialog.balance.titleString:			Balance",
"Xforge*monoDialog.balance.showValue:			True",
"Xforge*monoDialog.balance.scaleWidth:			200",
"Xforge*monoDialog.balance.orientation:			XmHORIZONTAL",
"Xforge*monoDialog.right.labelString:			Right",
"Xforge*monoDialog.apply.labelString:			Apply",
"Xforge*monoDialog.apply.marginHeight:			4",
"Xforge*monoDialog.apply.marginWidth:			4",
"Xforge*monoDialog.dismiss.labelString:			Dismiss",
"Xforge*monoDialog.dismiss.marginHeight:			4",
"Xforge*monoDialog.dismiss.marginWidth:			4",
"Xforge*monoDialog.help.labelString:			Help",
"Xforge*monoDialog.help.marginHeight:			4",
"Xforge*monoDialog.help.marginWidth:			4",
"Xforge*mixDialog.verticalSpacing:			10",
"Xforge*mixDialog.horizontalSpacing:			10",
"Xforge*mixDialog.autoUnmanage:				False",
"Xforge*mixDialog.mixWith.labelString:			Mix With...",
"Xforge*mixDialog.ratio.titleString:			Source Ratio",
"Xforge*mixDialog.ratio.showValue:			True",
"Xforge*mixDialog.ratio.scaleWidth:			200",
"Xforge*mixDialog.ratio.orientation:			XmHORIZONTAL",
"Xforge*mixDialog.auto.labelString:			Autogrow Buffer",
"Xforge*mixDialog.auto.set:				True",
"Xforge*mixDialog.apply.labelString:			Apply",
"Xforge*mixDialog.apply.marginHeight:			4",
"Xforge*mixDialog.apply.marginWidth:			4",
"Xforge*mixDialog.dismiss.labelString:			Dismiss",
"Xforge*mixDialog.dismiss.marginHeight:			4",
"Xforge*mixDialog.dismiss.marginWidth:			4",
"Xforge*mixDialog.help.labelString:			Help",
"Xforge*mixDialog.help.marginHeight:			4",
"Xforge*mixDialog.help.marginWidth:			4",
"Xforge*volDialog.verticalSpacing:			10",
"Xforge*volDialog.horizontalSpacing:			10",
"Xforge*volDialog.autoUnmanage:				False",
"Xforge*volDialog.rowCol.start.titleString:		Start (dB)",
"Xforge*volDialog.rowCol.start.showValue:		True",
"Xforge*volDialog.rowCol.start.scaleHeight:		200",
"Xforge*volDialog.rowCol.end.titleString:		End (dB)",
"Xforge*volDialog.rowCol.end.showValue:			True",
"Xforge*volDialog.rowCol.end.scaleHeight:		200",
"Xforge*volDialog.apply.labelString:			Apply",
"Xforge*volDialog.apply.marginHeight:			4",
"Xforge*volDialog.apply.marginWidth:			4",
"Xforge*volDialog.dismiss.labelString:			Dismiss",
"Xforge*volDialog.dismiss.marginHeight:			4",
"Xforge*volDialog.dissmis.marginWidth:			4",
"Xforge*volDialog.help.labelString:			Help",
"Xforge*volDialog.help.marginHeight:			4",
"Xforge*volDialog.help.marginWidth:			4",
"Xforge*echoDialog.verticalSpacing:			10",
"Xforge*echoDialog.horizontalSpacing:			10",
"Xforge*echoDialog.autoUnmanage:				False",
"Xforge*echoDialog.rowCol.gain.titleString:		Gain (dB)",
"Xforge*echoDialog.rowCol.gain.showValue:		True",
"Xforge*echoDialog.rowCol.gain.scaleHeight:		200",
"Xforge*echoDialog.rowCol.feedback.titleString:		Feedback (dB)",
"Xforge*echoDialog.rowCol.feedback.showValue:		True",
"Xforge*echoDialog.rowCol.feedback.scaleHeight:		200",
"Xforge*echoDialog.rowCol.delay.titleString:		Delay (sec)",
"Xforge*echoDialog.rowCol.delay.showValue:		True",
"Xforge*echoDialog.rowCol.delay.scaleHeight:		200",
"Xforge*echoDialog.apply.labelString:			Apply",
"Xforge*echoDialog.apply.marginHeight:			4",
"Xforge*echoDialog.apply.marginWidth:			4",
"Xforge*echoDialog.dismiss.labelString:			Dismiss",
"Xforge*echoDialog.dismiss.marginHeight:			4",
"Xforge*echoDialog.dismiss.marginWidth:			4",
"Xforge*echoDialog.help.labelString:			Help",
"Xforge*echoDialog.help.marginHeight:			4",
"Xforge*echoDialog.help.marginWidth:			4",
"Xforge*flanDialog.verticalSpacing:			10",
"Xforge*flanDialog.horizontalSpacing:			10",
"Xforge*flanDialog.autoUnmanage:				False",
"Xforge*flanDialog.rowCol.delay.titleString:		Delay (ms)",
"Xforge*flanDialog.rowCol.delay.showValue:		True",
"Xforge*flanDialog.rowCol.delay.scaleHeight:		200",
"Xforge*flanDialog.rowCol.depth.titleString:		Depth (ms)",
"Xforge*flanDialog.rowCol.depth.showValue:		True",
"Xforge*flanDialog.rowCol.depth.scaleHeight:		200",
"Xforge*flanDialog.rowCol.freq.titleString:		Frequency (Hz)",
"Xforge*flanDialog.rowCol.freq.showValue:		True",
"Xforge*flanDialog.rowCol.freq.scaleHeight:		200",
"Xforge*flanDialog.apply.labelString:			Apply",
"Xforge*flanDialog.apply.marginHeight:			4",
"Xforge*flanDialog.apply.marginWidth:			4",
"Xforge*flanDialog.dismiss.labelString:			Dismiss",
"Xforge*flanDialog.dismiss.marginHeight:			4",
"Xforge*flanDialog.dismiss.marginWidth:			4",
"Xforge*flanDialog.help.labelString:			Help",
"Xforge*flanDialog.help.marginHeight:			4",
"Xforge*flanDialog.help.marginWidth:			4",
"Xforge*revDialog.verticalSpacing:			10",
"Xforge*revDialog.horizontalSpacing:			10",
"Xforge*revDialog.autoUnmanage:				False",
"Xforge*revDialog.rowCol.dry.titleString:		Dry Out (dB)",
"Xforge*revDialog.rowCol.dry.showValue:			True",
"Xforge*revDialog.rowCol.dry.scaleHeight:		200",
"Xforge*revDialog.rowCol.wet.titleString:		Wet Out (dB)",
"Xforge*revDialog.rowCol.wet.showValue:			True",
"Xforge*revDialog.rowCol.wet.scaleHeight:		200",
"Xforge*revDialog.rowCol.damp.titleString:		Damping",
"Xforge*revDialog.rowCol.damp.showValue:			True",
"Xforge*revDialog.rowCol.damp.scaleHeight:		200",
"Xforge*revDialog.rowCol.fb.titleString:			Feedback (dB)",
"Xforge*revDialog.rowCol.fb.showValue:			True",
"Xforge*revDialog.rowCol.fb.scaleHeight:			200",
"Xforge*revDialog.apply.labelString:			Apply",
"Xforge*revDialog.apply.marginHeight:			4",
"Xforge*revDialog.apply.marginWidth:			4",
"Xforge*revDialog.dismiss.labelString:			Dismiss",
"Xforge*revDialog.dismiss.marginHeight:			4",
"Xforge*revDialog.dissmis.marginWidth:			4",
"Xforge*revDialog.help.labelString:			Help",
"Xforge*revDialog.help.marginHeight:			4",
"Xforge*revDialog.help.marginWidth:			4",
"Xforge*filtDialog.verticalSpacing:			10",
"Xforge*filtDialog.horizontalSpacing:			10",
"Xforge*filtDialog.autoUnmanage:				False",
"Xforge*filtDialog.rowCol.lowLabel.labelString:		Lower cutoff:",
"Xforge*filtDialog.rowCol.hiLabel.labelString:		Upper cutoff:",
"Xforge*filtDialog.rowCol.orderLabel.labelString:	Filter order:",
"Xforge*filtDialog.rowCol.typeLabel.labelString:		Filter type:",
"Xforge*filtDialog.rowCol.order.order1.labelString:	1",
"Xforge*filtDialog.rowCol.order.order2.labelString:	2",
"Xforge*filtDialog.rowCol.type.lp.labelString:		LP",
"Xforge*filtDialog.rowCol.type.hp.labelString:		HP",
"Xforge*filtDialog.rowCol.type.bp.labelString:		BP",
"Xforge*filtDialog.rowCol.type.br.labelString:		BR",
"Xforge*filtDialog.apply.labelString:			Apply",
"Xforge*filtDialog.apply.marginHeight:			4",
"Xforge*filtDialog.apply.marginWidth:			4",
"Xforge*filtDialog.dismiss.labelString:			Dismiss",
"Xforge*filtDialog.dismiss.marginHeight:			4",
"Xforge*filtDialog.dismiss.marginWidth:			4",
"Xforge*filtDialog.help.labelString:			Help",
"Xforge*filtDialog.help.marginHeight:			4",
"Xforge*filtDialog.help.marginWidth:			4",
"Xforge*compDialog.verticalSpacing:			10",
"Xforge*compDialog.horizontalSpacing:			10",
"Xforge*compDialog.autoUnmanage:				False",
"Xforge*compDialog.rowCol.dynamics.titleString:		Dynamics",
"Xforge*compDialog.rowCol.dynamics.showValue:		True",
"Xforge*compDialog.rowCol.dynamics.scaleHeight:		200",
"Xforge*compDialog.rowCol.velocity.titleString:		Velocity (Hz)",
"Xforge*compDialog.rowCol.velocity.showValue:		True",
"Xforge*compDialog.rowCol.velocity.scaleHeight:		200",
"Xforge*compDialog.apply.labelString:			Apply",
"Xforge*compDialog.apply.marginHeight:			4",
"Xforge*compDialog.apply.marginWidth:			4",
"Xforge*compDialog.dismiss.labelString:			Dismiss",
"Xforge*compDialog.dismiss.marginHeight:			4",
"Xforge*compDialog.dismiss.marginWidth:			4",
"Xforge*compDialog.help.labelString:			Help",
"Xforge*compDialog.help.marginHeight:			4",
"Xforge*compDialog.help.marginWidth:			4",
"Xforge*licenseDialog.dialogTitle:			Xforge license",
"Xforge*licenseDialog.verticalSpacing:			10",
"Xforge*licenseDialog.horizontalSpacing:			10",
"Xforge*licenseDialog*licenseText.fontList:				"
"	-*-courier-medium-r-normal--14-100-*-*-*-*-*",
"Xforge*licenseDialog*licenseText.height:		500",
"Xforge*licenseDialog*licenseText.width:			700",
"Xforge*licenseDialog.dismiss.labelString:		Dismiss",
"Xforge*licenseDialog.dismiss.marginHeight:		4",
"Xforge*licenseDialog.dismiss.marginWidth:		4",
"Xforge*helpDialog.dialogTitle:				Xforge help",
"Xforge*helpDialog.verticalSpacing:			10",
"Xforge*helpDialog.horizontalSpacing:			10",
"Xforge*helpDialog*helpText.fontList:				"
"	-*-helvetica-bold-r-normal--*-120-*-*-*-*-*",
"Xforge*helpDialog*helpText.height:			400",
"Xforge*helpDialog*helpText.width:			500",
"Xforge*helpDialog.dismiss.labelString:			Dismiss",
"Xforge*helpDialog.dismiss.marginHeight:			4",
"Xforge*helpDialog.dismiss.marginWidth:			4",
"Xforge*menuBar*file.labelString:			File",
"Xforge*menuBar*file.mnemonic:				F",
"Xforge*menuBar*file.new.labelString:			New",
"Xforge*menuBar*file.new.mnemonic:			N",
"Xforge*menuBar*file.open.labelString:			Open...",
"Xforge*menuBar*file.open.mnemonic:			O",
"Xforge*menuBar*file.open.accelerator:			Ctrl<Key>O",
"Xforge*menuBar*file.open.acceleratorText:		Ctrl+O",
"Xforge*menuBar*file.save.labelString:			Save",
"Xforge*menuBar*file.save.mnemonic:			S",
"Xforge*menuBar*file.save.accelerator:			Ctrl<Key>S",
"Xforge*menuBar*file.save.acceleratorText:		Ctrl+S",
"Xforge*menuBar*file.saveAs.labelString:			Save As...",
"Xforge*menuBar*file.saveAs.mnemonic:			A",
"Xforge*menuBar*file.revert.labelString:			Revert",
"Xforge*menuBar*file.revert.mnemonic:			R",
"Xforge*menuBar*file.exit.labelString:			Exit",
"Xforge*menuBar*file.exit.mnemonic:			x",
"Xforge*menuBar*file.exit.accelerator:			Ctrl<Key>Q",
"Xforge*menuBar*file.exit.acceleratorText:		Ctrl+Q",
"Xforge*menuBar*edit.labelString:			Edit",
"Xforge*menuBar*edit.mnemonic:				E",
"Xforge*menuBar*edit.undo.labelString:			Undo",
"Xforge*menuBar*edit.undo.mnemonic:			U",
"Xforge*menuBar*edit.undo.accelerator:			Alt<Key>BackSpace",
"Xforge*menuBar*edit.undo.acceleratorText:		Alt+BS",
"Xforge*menuBar*edit.copy.labelString:			Copy",
"Xforge*menuBar*edit.copy.mnemonic:			C",
"Xforge*menuBar*edit.copy.accelerator:			Ctrl<Key>Insert",
"Xforge*menuBar*edit.copy.acceleratorText:		Ctrl+Ins",
"Xforge*menuBar*edit.cut.labelString:			Cut",
"Xforge*menuBar*edit.cut.mnemonic:			t",
"Xforge*menuBar*edit.cut.accelerator:			Shift<Key>Delete",
"Xforge*menuBar*edit.cut.acceleratorText:		Shift+Del",
"Xforge*menuBar*edit.paste.labelString:			Paste",
"Xforge*menuBar*edit.paste.mnemonic:			P",
"Xforge*menuBar*edit.paste.accelerator:			Shift<Key>Insert",
"Xforge*menuBar*edit.paste.acceleratorText:		Shift+Ins",
"Xforge*menuBar*edit.swap.labelString:			Swap To Clip",
"Xforge*menuBar*edit.swap.mnemonic:			S",
"Xforge*menuBar*edit.clear.labelString:			Clear",
"Xforge*menuBar*edit.clear.mnemonic:			e",
"Xforge*menuBar*edit.delete.labelString:			Delete",
"Xforge*menuBar*edit.delete.mnemonic:			D",
"Xforge*menuBar*edit.delete.accelerator:			<Key>BackSpace",
"Xforge*menuBar*edit.delete.acceleratorText:		BS",
"Xforge*menuBar*edit.selectAll.labelString:		Select All",
"Xforge*menuBar*edit.selectAll.mnemonic:			a",
"Xforge*menuBar*edit.selectAll.accelerator:		Ctrl<Key>/",
"Xforge*menuBar*edit.selectAll.acceleratorText:		Ctrl+/	",
"Xforge*menuBar*edit.selectLeft.labelString:		Select Left",
"Xforge*menuBar*edit.selectLeft.mnemonic:		l",
"Xforge*menuBar*edit.selectRight.labelString:		Select Right",
"Xforge*menuBar*edit.selectRight.mnemonic:		r",
"Xforge*menuBar*edit.deselect.labelString:		Deselect All",
"Xforge*menuBar*edit.deselect.mnemonic:			s",
"Xforge*menuBar*edit.deselect.accelerator:		Ctrl<Key>\\\\",
"Xforge*menuBar*edit.deselect.acceleratorText:		Ctrl+\\\\",
"Xforge*menuBar*edit.beginData.labelString:		Begin Data",
"Xforge*menuBar*edit.beginData.mnemonic:			B",
"Xforge*menuBar*edit.beginData.accelerator:		Ctrl<Key>Home",
"Xforge*menuBar*edit.beginData.acceleratorText:		Ctrl+Home",
"Xforge*menuBar*edit.extBegin.labelString:		Extend Begin Data",
"Xforge*menuBar*edit.extBegin.mnemonic:			x",
"Xforge*menuBar*edit.extBegin.accelerator:		Ctrl Shift<Key>Home",
"Xforge*menuBar*edit.extBegin.acceleratorText:		Ctrl+Shift+Home",
"Xforge*menuBar*edit.endData.labelString:		End Data",
"Xforge*menuBar*edit.endData.mnemonic:			E",
"Xforge*menuBar*edit.endData.accelerator:		Ctrl<Key>End",
"Xforge*menuBar*edit.endData.acceleratorText:		Ctrl+End",
"Xforge*menuBar*edit.extEnd.labelString:			Extend End Data",
"Xforge*menuBar*edit.extEnd.mnemonic:			e",
"Xforge*menuBar*edit.extEnd.accelerator:			Ctrl Shift<Key>End",
"Xforge*menuBar*edit.extEnd.acceleratorText:		Ctrl+Shift+End",
"Xforge*menuBar*edit.removeDC.labelString:		Remove DC",
"Xforge*menuBar*edit.removeDC.mnemonic:			R",
"Xforge*menuBar*edit.addWS.labelString:			Add Workspace...",
"Xforge*menuBar*edit.addWS.mnemonic:			W",
"Xforge*menuBar*view.labelString:			View",
"Xforge*menuBar*view.mnemonic:				V",
"Xforge*menuBar*view.showRange.labelString:		Show Range",
"Xforge*menuBar*view.showRange.mnemonic:			S",
"Xforge*menuBar*view.showAll.labelString:		Show All",
"Xforge*menuBar*view.showAll.mnemonic:			a",
"Xforge*menuBar*view.zoomOut.labelString:		Zoom Out",
"Xforge*menuBar*view.zoomOut.mnemonic:			Z",
"Xforge*menuBar*view.closeView.labelString:		Close View",
"Xforge*menuBar*view.closeView.mnemonic:			C",
"Xforge*menuBar*view.refresh.labelString:		Refresh",
"Xforge*menuBar*view.refresh.mnemonic:			R",
"Xforge*menuBar*view.refresh.accelerator:		Ctrl<Key>L",
"Xforge*menuBar*view.refresh.acceleratorText:		Ctrl+L",
"Xforge*menuBar*sound.labelString:			Sound",
"Xforge*menuBar*sound.mnemonic:				S",
"Xforge*menuBar*sound.playRange.labelString:		Play Range",
"Xforge*menuBar*sound.playRange.mnemonic:		P",
"Xforge*menuBar*sound.playRange.accelerator:		Ctrl<Key>R",
"Xforge*menuBar*sound.playRange.acceleratorText:		Ctrl+R",
"Xforge*menuBar*sound.playAll.labelString:		Play All",
"Xforge*menuBar*sound.playAll.mnemonic:			a",
"Xforge*menuBar*sound.playAll.accelerator:		Ctrl<Key>P",
"Xforge*menuBar*sound.playAll.acceleratorText:		Ctrl+P",
"Xforge*menuBar*sound.stop.labelString:			Stop",
"Xforge*menuBar*sound.stop.mnemonic:			S",
"Xforge*menuBar*sound.loop.labelString:			Loop",
"Xforge*menuBar*sound.loop.mnemonic:			L",
"Xforge*menuBar*sound.loop.set:				False",
"Xforge*menuBar*sound.loop.visibleWhenOff:		True",
"Xforge*menuBar*sound.sampler.labelString:		Sampler...",
"Xforge*menuBar*sound.sampler.mnemonic:			m",
"Xforge*menuBar*sound.params.labelString:		Params...",
"Xforge*menuBar*sound.params.mnemonic:			r",
"Xforge*menuBar*sound.resample.labelString:		Resample...",
"Xforge*menuBar*sound.resample.mnemonic:			R",
"Xforge*menuBar*sound.toMono.labelString:		Convert To Mono...",
"Xforge*menuBar*sound.toMono.mnemonic:			o",
"Xforge*menuBar*sound.toStereo.labelString:		Convert To Stereo",
"Xforge*menuBar*sound.toStereo.mnemonic:			t",
"Xforge*menuBar*sound.invert.labelString:		Phase Invert",
"Xforge*menuBar*sound.invert.mnemonic:			I",
"Xforge*menuBar*sound.reverse.labelString:		Reverse",
"Xforge*menuBar*sound.reverse.mnemonic:			e",
"Xforge*menuBar*sound.mix.labelString:			Mix...",
"Xforge*menuBar*sound.mix.mnemonic:			M",
"Xforge*menuBar*effects.labelString:			Effects",
"Xforge*menuBar*effects.mnemonic:			c",
"Xforge*menuBar*effects.volume.labelString:		Volume...",
"Xforge*menuBar*effects.volume.mnemonic:			V",
"Xforge*menuBar*effects.echo.labelString:		Echo...",
"Xforge*menuBar*effects.echo.mnemonic:			E",
"Xforge*menuBar*effects.reverb.labelString:		Reverb...",
"Xforge*menuBar*effects.reverb.mnemonic:			R",
"Xforge*menuBar*effects.flange.labelString:		Flange...",
"Xforge*menuBar*effects.flange.mnemonic:			l",
"Xforge*menuBar*effects.filter.labelString:		Filter...",
"Xforge*menuBar*effects.filter.mnemonic:			F",
"Xforge*menuBar*effects.compress.labelString:		Compress...",
"Xforge*menuBar*effects.compress.mnemonic:		C",
"Xforge*menuBar*options.labelString:			Options",
"Xforge*menuBar*options.mnemonic:			O",
"Xforge*menuBar*options.okPrompts.labelString:		Ok Prompts",
"Xforge*menuBar*options.okPrompts.mnemonic:		O",
"Xforge*menuBar*options.okPrompts.set:			True",
"Xforge*menuBar*options.okPrompts.visibleWhenOff:	True",
"Xforge*menuBar*options.saveOpts.labelString:		Save Options",
"Xforge*menuBar*options.saveOpts.mnemonic:		S",
"Xforge*menuBar*help.labelString:			Help",
"Xforge*menuBar*help.mnemonic:				H",
"Xforge*menuBar*help.about.labelString:			About...",
"Xforge*menuBar*help.about.mnemonic:			A",
"Xforge*menuBar*help.license.labelString:		License...",
"Xforge*menuBar*help.license.mnemonic:			L",
0 };
