/*
	$Id: version.h,v 1.5 2001/09/03 18:54:15 rogue Exp $

	Copyright (c) 1999 - 2001 Xforge project
*/

#ifndef _VERSION_H
#define _VERSION_H

#define XFORGE_VERSION "0.2.2"

#define ABOUT_STRING_1 "Xforge " XFORGE_VERSION " - a Motif wave editor"
#define ABOUT_STRING_2 "Copyright (c) 1999 - 2001 Xforge project"
#define ABOUT_STRING_3 "Xforge is copyrighted software;"
#define ABOUT_STRING_4 "Please see License in Help menu"

#endif /* !_VERSION_H */
