#!/usr/bin/sed
# $Id: adconv.sed,v 1.1 2000/04/21 22:28:20 rogue Exp $

s/Alt<Key>BackSpace$/Ctrl<Key>Z/
s/Alt+BS$/Ctrl+Z/
s/Ctrl<Key>Insert$/Ctrl<Key>C/
s/Ctrl+Ins$/Ctrl+C/
s/Shift<Key>Delete$/Ctrl<Key>X/
s/Shift+Del$/Ctrl+X/
s/Shift<Key>Insert$/Ctrl<Key>V/
s/Shift+Ins$/Ctrl+V/
s/<Key>BackSpace$/<Key>Delete/
s/BS$/Del/
s/Ctrl<Key>\/$/Ctrl<Key>A/
s/Ctrl+\/$/Ctrl+A/
s/Ctrl<Key>\\\\$/Shift Ctrl<Key>A/
s/Ctrl+\\\\$/Shift+Ctrl+A/
s/Ctrl<Key>Home$/<Key>Home/
s/Ctrl+Home$/Home/
s/Ctrl Shift<Key>Home$/Shift<Key>Home/
s/Ctrl+Shift+Home$/Shift+Home/
s/Ctrl<Key>End$/<Key>End/
s/Ctrl+End$/End/
s/Ctrl Shift<Key>End$/Shift<Key>End/
s/Ctrl+Shift+End$/Shift+End/
