/*
	$Id: bufutil.h,v 1.2 2001/03/01 22:49:46 rogue Exp $

	Copyright (c) 1999 - 2001 Xforge project
*/

#ifndef _BUFUTIL_H
#define _BUFUTIL_H

extern struct buffer *root;
extern int nbuf;

/* bufutil.c prototypes */
extern void setbufname(struct buffer *, const char *);
extern struct buffer *add_buf(void);
extern void remove_buf(struct buffer *);
extern int undo_create(struct buffer *);
extern void undo(struct buffer *);
extern void buf_remdc(struct buffer *);
extern int buf_addws(struct buffer *, int);
extern void buf_invert(struct buffer *);
extern void buf_reverse(struct buffer *);
extern void buf_tomono(struct buffer *, float);
extern void buf_tostereo(struct buffer *);
extern int buf_mtosshuffle(struct buffer *);
extern int buf_stomshuffle(struct buffer *);
extern int buf_mix(struct buffer *, struct buffer *, float, int);

#endif /* !_BUFUTIL_H */
